#!/usr/bin/env bash
#!/bin/env bash

## Name: RNAprof.sh
## Version: 1.2.0
## Date: Jan 01, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Execute RNAprof for all pairs of experiments


ECHO_E="echo -e"
PREFIX="."
MAX=1
LEN=50
GP=1e-3
LP=1e-3

usage() {
    $ECHO_E "Usage: ./RNAprof.sh [OPTIONS]\n"
    $ECHO_E "Optional:"
    $ECHO_E "-C|--CPU MAX"
    $ECHO_E "\tRun up to MAX processes in parallel. By default, MAX=1."
    $ECHO_E "-g|--globalP GP"
    $ECHO_E "\tLoci with p-value less than GP are shown in figures. By default, GP=1e-3."
    $ECHO_E "-h|-?|--help"
    $ECHO_E "\tPrint this help."
    $ECHO_E "-l|--localP LP"
    $ECHO_E "\tNucleotide-based differential expression with p-value less than LP is considered significant. By default, LP=1e-3."
    $ECHO_E "-L|--length LEN"
    $ECHO_E "\tTranscript shorter than this length threshold is discarded. Consider loci of at least LEN/2 bp. By default, LEN=50." 
    $ECHO_E "-p|--prefix PREFIX"
    $ECHO_E "\tThis is the same directory as PREFIX given in configuration step. All generated scripts are located in PREFIX/script. By default,PREFIX=.\n"
    exit 1
}

if [ ! $1 ]
then
    usage
fi

while :
do
    case $1 in
	-h | --help | -\?)
	    usage
	    ;;
	-g | --globalP)
	    GP=$2
	    shift 2
	    ;;
	--globalP=*)
            GP=${1#*=} #delete everything up till '='
            shift
            ;;
	-l | --localP)
	    LP=$2
	    shift 2
	    ;;
	--localP=*)
            LP=${1#*=} #delete everything up till '='
            shift
            ;;
	-L | --length)
	    LEN=$2
	    shift 2
	    ;;
	--length=*)
            LEN=${1#*=} #delete everything up till '='
            shift
            ;;
	-p | --prefix)
	    PREFIX=$2
	    shift 2
	    ;;
	--prefix=*)
            PREFIX=${1#*=} #delete everything up till '='
            shift
            ;;
	-C | --CPU)
	    MAX=$2
	    shift 2
	    ;;
	--CPU=*)
            MAX=${1#*=}
            shift
            ;;
	-*)
	    $ECHO_ "Unknown option: $1" >&2
	    shift
	    ;;
	--) #end of all options
	    shift
	    break
	    ;;
	*) #no more options, stop while loop
	    break
	    ;;
    esac
done

## directory containing sources
SRC=$0
SRC=`dirname $SRC`
cd $SRC
SRC=$PWD
cd - > /dev/null

## change current directory to $PREFIX
cd $PREFIX
PREFIX=$PWD

$ECHO_E "\nRNAPROF..."

rm -rf $PREFIX/result/*

## run in parallel coverage_*
find $PREFIX/script -name rnaprof_\*.sh | xargs -i --max-procs=$MAX bash -c "bash {} $LEN $GP $LP"

if [ $? -ne 0 ]
then
    echo "Errors while running $PREFIX/script/rnaprof*"
    exit 1
fi

## concatenate results
for i in $PREFIX/result/rnaprof*forpanr*
do
    parenti=${i%forpanr*}
    if [ ! -d "$parenti" ]
    then
	mkdir $parenti
    fi
    touch $parenti/loci.gff

    numpng=`ls $i | grep .png | wc -l`
    if [ $numpng -gt 0 ]
    then
	mv $i/*png $parenti
    fi
    numgff=`ls $i | grep .gff | wc -l`
    if [ $numgff -gt 0 ]
    then
	cat $i/*gff >> $parenti/loci.gff
    fi
    numsft=`ls $i | grep .sft | wc -l`
    if [ $numsft -gt 0 ]
    then
	head -1 $i/*sft > $parenti/header.sft
	tail -n +2 $i/*sft >> $parenti/sizefactors.sft
    fi
    numtxt=`ls $i | grep .txt | wc -l`
    if [ $numtxt -gt 0 ]
    then
	numtsc=`grep "Number of genes:" $i/*txt | sed s/"Number of genes: "//g`
	echo $numtsc >> $parenti/numtsc.num
	
	numeve=`grep "Number of events:" $i/*txt | sed s/"Number of events: "//g`
	echo $numeve >> $parenti/numeve.num
	
	numtsceve=`grep "Number of genes with events:" $i/*txt | sed s/"Number of genes with events: "//g`
	echo $numtsceve >> $parenti/numtsceve.num
	
	grep "Number of reads" $i/*txt > $parenti/report.txt
    else
	touch $parenti/numtsc.num
	touch $parenti/numeve.num
	touch $parenti/numtsceve.num
	touch $parenti/report.txt
    fi
    rm -r $i
done
for parenti in $PREFIX/result/*
do
    sum=0
    while read line
    do
	sum=`expr $sum + $line`
    done < $parenti/numtsc.num
    $ECHO_E "Number of genes: $sum" >> $parenti/report.txt

    rm $parenti/*.num
    cat $parenti/header.sft $parenti/sizefactors.sft > $parenti/sizefactors.txt
    rm $parenti/*.sft
done

## add adjusted p-values
for parenti in $PREFIX/result/*
do
    cd $parenti
    if [ -s loci.gff ]
    then
	Rscript $SRC/R/adjustP.R --input=loci.gff --adjust=BH --padj=$GP --output=loci_adj.gff
	mv loci_adj.gff loci.gff
        cat reportadj.txt >> report.txt
	rm reportadj.txt
    else
	echo "Number of events: 0" >> report.txt
	echo "Number of genes with events: 0" >> report.txt
    fi
    cd - > /dev/null
done
$ECHO_E "\nDone."
